CodeMirror.defineMode("clike", function(config, parserConfig) {
  var indentUnit = config.indentUnit,
      keywords = parserConfig.keywords || {},
      blockKeywords = parserConfig.blockKeywords || {},
      atoms = parserConfig.atoms || {},
      hooks = parserConfig.hooks || {},
      multiLineStrings = parserConfig.multiLineStrings;
  var isOperatorChar = /[+\-*&%=<>!?|\/]/;

  var curPunc;

  function tokenBase(stream, state) {
    var ch = stream.next();
    if (hooks[ch]) {
      var result = hooks[ch](stream, state);
      if (result !== false) return result;
    }
    if (ch == '"' || ch == "'") {
      state.tokenize = tokenString(ch);
      return state.tokenize(stream, state);
    }
    if (/[\[\]{}\(\),;\:\.]/.test(ch)) {
      curPunc = ch;
      return null
    }
    if (/\d/.test(ch)) {
      stream.eatWhile(/[\w\.]/);
      return "number";
    }
    if (ch == "/") {
      if (stream.eat("*")) {
        state.tokenize = tokenComment;
        return tokenComment(stream, state);
      }
      if (stream.eat("/")) {
        stream.skipToEnd();
        return "comment";
      }
    }
    if (isOperatorChar.test(ch)) {
      stream.eatWhile(isOperatorChar);
      return "operator";
    }
    stream.eatWhile(/[\w\$_]/);
    var cur = stream.current();
    if (keywords.propertyIsEnumerable(cur)) {
      if (blockKeywords.propertyIsEnumerable(cur)) curPunc = "newstatement";
      return "keyword";
    }
    if (atoms.propertyIsEnumerable(cur)) return "atom";
    return "word";
  }

  function tokenString(quote) {
    return function(stream, state) {
      var escaped = false, next, end = false;
      while ((next = stream.next()) != null) {
        if (next == quote && !escaped) {end = true; break;}
        escaped = !escaped && next == "\\";
      }
      if (end || !(escaped || multiLineStrings))
        state.tokenize = tokenBase;
      return "string";
    };
  }

  function tokenComment(stream, state) {
    var maybeEnd = false, ch;
    while (ch = stream.next()) {
      if (ch == "/" && maybeEnd) {
        state.tokenize = tokenBase;
        break;
      }
      maybeEnd = (ch == "*");
    }
    return "comment";
  }

  function Context(indented, column, type, align, prev) {
    this.indented = indented;
    this.column = column;
    this.type = type;
    this.align = align;
    this.prev = prev;
  }
  function pushContext(state, col, type) {
    return state.context = new Context(state.indented, col, type, null, state.context);
  }
  function popContext(state) {
    var t = state.context.type;
    if (t == ")" || t == "]" || t == "}")
      state.indented = state.context.indented;
    return state.context = state.context.prev;
  }

  // Interface

  return {
    startState: function(basecolumn) {
      return {
        tokenize: null,
        context: new Context((basecolumn || 0) - indentUnit, 0, "top", false),
        indented: 0,
        startOfLine: true
      };
    },

    token: function(stream, state) {
      var ctx = state.context;
      if (stream.sol()) {
        if (ctx.align == null) ctx.align = false;
        state.indented = stream.indentation();
        state.startOfLine = true;
      }
      if (stream.eatSpace()) return null;
      curPunc = null;
      var style = (state.tokenize || tokenBase)(stream, state);
      if (style == "comment" || style == "meta") return style;
      if (ctx.align == null) ctx.align = true;

      if ((curPunc == ";" || curPunc == ":") && ctx.type == "statement") popContext(state);
      else if (curPunc == "{") pushContext(state, stream.column(), "}");
      else if (curPunc == "[") pushContext(state, stream.column(), "]");
      else if (curPunc == "(") pushContext(state, stream.column(), ")");
      else if (curPunc == "}") {
        while (ctx.type == "statement") ctx = popContext(state);
        if (ctx.type == "}") ctx = popContext(state);
        while (ctx.type == "statement") ctx = popContext(state);
      }
      else if (curPunc == ctx.type) popContext(state);
      else if (ctx.type == "}" || ctx.type == "top" || (ctx.type == "statement" && curPunc == "newstatement"))
        pushContext(state, stream.column(), "statement");
      state.startOfLine = false;
      return style;
    },

    indent: function(state, textAfter) {
      if (state.tokenize != tokenBase && state.tokenize != null) return 0;
      var ctx = state.context, firstChar = textAfter && textAfter.charAt(0);
      if (ctx.type == "statement" && firstChar == "}") ctx = ctx.prev;
      var closing = firstChar == ctx.type;
      if (ctx.type == "statement") return ctx.indented + (firstChar == "{" ? 0 : indentUnit);
      else if (ctx.align) return ctx.column + (closing ? 0 : 1);
      else return ctx.indented + (closing ? 0 : indentUnit);
    },

    electricChars: "{}"
  };
});

(function() {
  function words(str) {
    var obj = {}, words = str.split(" ");
    for (var i = 0; i < words.length; ++i) obj[words[i]] = true;
    return obj;
  }
  var cKeywords = "auto if break int case long char register continue return default short do sizeof " +
    "double static else struct entry switch extern typedef float union for unsigned " +
    "goto while enum void const signed volatile";

  function cppHook(stream, state) {
    if (!state.startOfLine) return false;
    stream.skipToEnd();
    return "meta";
  }

  // C#-style strings where "" escapes a quote.
  function tokenAtString(stream, state) {
    var next;
    while ((next = stream.next()) != null) {
      if (next == '"' && !stream.eat('"')) {
        state.tokenize = null;
        break;
      }
    }
    return "string";
  }

  CodeMirror.defineMIME("text/x-csrc", {
    name: "clike",
    keywords: words(cKeywords),
    blockKeywords: words("case do else for if switch while struct"),
    atoms: words("null"),
    hooks: {"#": cppHook}
  });
  CodeMirror.defineMIME("text/x-c++src", {
    name: "clike",
    keywords: words(cKeywords + " asm dynamic_cast namespace reinterpret_cast try bool explicit new " +
                    "static_cast typeid catch operator template typename class friend private " +
                    "this using const_cast inline public throw virtual delete mutable protected " +
                    "wchar_t"),
    blockKeywords: words("catch class do else finally for if struct switch try while"),
    atoms: words("true false null"),
    hooks: {"#": cppHook}
  });
  CodeMirror.defineMIME("text/x-java", {
    name: "clike",
    keywords: words("abstract assert boolean break byte case catch char class const continue default " + 
                    "do double else enum extends final finally float for goto if implements import " +
                    "instanceof int interface long native new package private protected public " +
                    "return short static strictfp super switch synchronized this throw throws transient " +
                    "try void volatile while"),
    blockKeywords: words("catch class do else finally for if switch try while"),
    atoms: words("true false null"),
    hooks: {
      "@": function(stream, state) {
        stream.eatWhile(/[\w\$_]/);
        return "meta";
      }
    }
  });
  CodeMirror.defineMIME("text/x-csharp", {
    name: "clike",
    keywords: words("abstract as base bool break byte case catch char checked class const continue decimal" + 
                    " default delegate do double else enum event explicit extern finally fixed float for" + 
                    " foreach goto if implicit in int interface internal is lock long namespace new object" + 
                    " operator out override params private protected public readonly ref return sbyte sealed short" + 
                    " sizeof stackalloc static string struct switch this throw try typeof uint ulong unchecked" + 
                    " unsafe ushort using virtual void volatile while add alias ascending descending dynamic from get" + 
                    " global group into join let orderby partial remove select set value var yield"),
    blockKeywords: words("catch class do else finally for foreach if struct switch try while"),
    atoms: words("true false null AccessViolationException ActivationContext Activator AggregateException AppDomain AppDomainManager AppDomainSetup AppDomainUnloadedException ApplicationException ApplicationId ApplicationIdentity ArgumentException ArgumentNullException ArgumentOutOfRangeException ArithmeticException Array ArrayTypeMismatchException AssemblyLoadEventArgs Attribute AttributeUsageAttribute BadImageFormatException BitConverter Buffer CannotUnloadAppDomainException CharEnumerator CLSCompliantAttribute Console ConsoleCancelEventArgs ContextBoundObject ContextMarshalException ContextStaticAttribute Convert DataMisalignedException DBNull Delegate DivideByZeroException DllNotFoundException DuplicateWaitObjectException EntryPointNotFoundException Enum Environment EventArgs Exception ExecutionEngineException FieldAccessException FileStyleUriParser FlagsAttribute FormatException FtpStyleUriParser GC GenericUriParser GopherStyleUriParser HttpStyleUriParser IndexOutOfRangeException InsufficientExecutionStackException InsufficientMemoryException InvalidCastException InvalidOperationException InvalidProgramException InvalidTimeZoneException Lazy LdapStyleUriParser LoaderOptimizationAttribute LocalDataStoreSlot MarshalByRefObject Math MemberAccessException MethodAccessException MissingFieldException MissingMemberException MissingMethodException MTAThreadAttribute MulticastDelegate MulticastNotSupportedException NetPipeStyleUriParser NetTcpStyleUriParser NewsStyleUriParser NonSerializedAttribute NotFiniteNumberException NotImplementedException NotSupportedException Nullable NullReferenceException Object ObjectDisposedException ObsoleteAttribute OperatingSystem OperationCanceledException OutOfMemoryException OverflowException ParamArrayAttribute PlatformNotSupportedException Random RankException ResolveEventArgs SerializableAttribute StackOverflowException STAThreadAttribute String StringComparer SystemException ThreadStaticAttribute TimeoutException TimeZone TimeZoneInfo TimeZoneInfo.AdjustmentRule TimeZoneNotFoundException Tuple Type TypeAccessException TypeInitializationException TypeLoadException TypeUnloadedException UnauthorizedAccessException UnhandledExceptionEventArgs Uri UriBuilder UriFormatException UriParser UriTemplate UriTemplateEquivalenceComparer UriTemplateMatch UriTemplateMatchException UriTemplateTable UriTypeConverter ValueType Version WeakReference Structure ArgIterator ArraySegment Boolean Byte Char ConsoleKeyInfo DateTime DateTimeOffset Decimal Double Guid Int16 Int32 Int64 IntPtr ModuleHandle Nullable RuntimeArgumentHandle RuntimeFieldHandle RuntimeMethodHandle RuntimeTypeHandle SByte Single TimeSpan TimeZoneInfo.TransitionTime TypedReference UInt16 UInt32 UInt64 UIntPtr Void _AppDomain IAppDomainSetup IAsyncResult ICloneable IComparable IComparable IConvertible ICustomFormatter IDisposable IEquatable IFormatProvider IFormattable IObservable IObserver IServiceProvider Action Action AppDomainInitializer AssemblyLoadEventHandler AsyncCallback Comparison ConsoleCancelEventHandler Converter CrossAppDomainDelegate EventHandler EventHandler Func Predicate ResolveEventHandler UnhandledExceptionEventHandler ActivationContext.ContextForm AppDomainManagerInitializationOptions AttributeTargets Base64FormattingOptions ConsoleColor ConsoleKey ConsoleModifiers ConsoleSpecialKey DateTimeKind DayOfWeek Environment.SpecialFolder Environment.SpecialFolderOption EnvironmentVariableTarget GCCollectionMode GCNotificationStatus GenericUriParserOptions LoaderOptimization MidpointRounding PlatformID StringComparison StringSplitOptions TypeCode UriComponents UriFormat UriHostNameType UriIdnScope UriKind UriPartial Comparer Dictionary Dictionary Dictionary EqualityComparer HashSet KeyedByTypeCollection KeyNotFoundException LinkedList LinkedListNode List Queue SortedDictionary SortedDictionary SortedDictionary SortedList SortedSet Stack SynchronizedCollection SynchronizedKeyedCollection SynchronizedReadOnlyCollection Dictionary Dictionary Dictionary HashSet KeyValuePair LinkedList List Queue SortedDictionary SortedDictionary SortedDictionary " +
	"SortedSet Stack Interface ICollection IComparer IDictionary IEnumerable IEnumerator IEqualityComparer IList ISet Enumerable EnumerableExecutor EnumerableExecutor EnumerableQuery EnumerableQuery Lookup OrderedParallelQuery ParallelEnumerable ParallelQuery ParallelQuery Queryable Interface IGrouping ILookup IOrderedEnumerable IOrderedQueryable IOrderedQueryable IQueryable IQueryable IQueryProvider ParallelExecutionMode ParallelMergeOptions AspNetHostingPermission AspNetHostingPermissionAttribute DefaultHttpHandler HtmlString HttpApplication HttpApplicationState HttpApplicationStateBase HttpApplicationStateWrapper HttpBrowserCapabilities HttpBrowserCapabilitiesBase HttpBrowserCapabilitiesWrapper HttpCachePolicy HttpCachePolicyBase HttpCachePolicyWrapper HttpCacheVaryByContentEncodings HttpCacheVaryByHeaders HttpCacheVaryByParams HttpClientCertificate HttpCompileException HttpContext HttpContextBase HttpContextWrapper HttpCookie HttpCookieCollection HttpException HttpFileCollection HttpFileCollectionBase HttpFileCollectionWrapper HttpModuleCollection HttpParseException HttpPostedFile HttpPostedFileBase HttpPostedFileWrapper HttpRequest HttpRequestBase HttpRequestValidationException HttpRequestWrapper HttpResponse HttpResponseBase HttpResponseWrapper HttpRuntime HttpServerUtility HttpServerUtilityBase HttpServerUtilityWrapper HttpSessionStateBase HttpSessionStateWrapper HttpStaticObjectsCollection HttpStaticObjectsCollectionBase HttpStaticObjectsCollectionWrapper HttpUnhandledException HttpUtility HttpWorkerRequest HttpWriter IisTraceListener ParserError ParserErrorCollection PreApplicationStartMethodAttribute ProcessInfo ProcessModelInfo SiteMap SiteMapNode SiteMapNodeCollection SiteMapProvider SiteMapProviderCollection SiteMapResolveEventArgs StaticSiteMapProvider TraceContext TraceContextEventArgs TraceContextRecord VirtualPathUtility WebPageTraceListener XmlSiteMapProvider IHtmlString IHttpAsyncHandler IHttpHandler IHttpHandlerFactory IHttpModule IPartitionResolver BeginEventHandler EndEventHandler HttpCacheValidateHandler HttpResponseSubstitutionCallback HttpWorkerRequest.EndOfSendNotification SiteMapResolveEventHandler TraceContextEventHandler ApplicationShutdownReason AspNetHostingPermissionLevel HttpCacheability HttpCacheRevalidation HttpCookieMode HttpValidationStatus ProcessShutdownReason ProcessStatus RequestNotification RequestNotificationStatus TraceMode AsyncPostBackErrorEventArgs AsyncPostBackTrigger AttributeCollection AuthenticationServiceManager BaseParser BasePartialCachingControl BaseTemplateParser BindableTemplateBuilder BoundPropertyEntry BuilderPropertyEntry ChtmlTextWriter ClientScriptManager CompiledBindableTemplateBuilder CompiledTemplateBuilder ComplexPropertyEntry CompositeScriptReference CompositeScriptReferenceEventArgs ConstructorNeedsTagAttribute Control ControlBuilder ControlBuilderAttribute ControlCachePolicy ControlCollection ControlSkin ControlValuePropertyAttribute CssClassPropertyAttribute CssStyleCollection DataBinder DataBinding DataBindingCollection DataBindingHandlerAttribute DataBoundLiteralControl DataControlExtensions DataKeyPropertyAttribute DataSourceCacheDurationConverter DataSourceControl DataSourceControlBuilder DataSourceSelectArguments DataSourceView DesignerDataBoundLiteralControl DesignTimeParseData DesignTimeTemplateParser EmptyControlCollection EventEntry ExpressionBinding ExpressionBindingCollection ExtenderControl FileLevelControlBuilderAttribute FileLevelMasterPageControlBuilder FileLevelPageControlBuilder FileLevelUserControlBuilder FilterableAttribute HiddenFieldPageStatePersister HierarchicalDataSourceControl HierarchicalDataSourceView HistoryEventArgs Html32TextWriter HtmlTextWriter IDReferencePropertyAttribute ImageClickEventArgs IndexedString ListSourceHelper LiteralControl LosFormatter MasterPage MasterPageControlBuilder NonVisualControlAttribute ObjectConverter ObjectPersistData ObjectStateFormatter ObjectTagBuilder OutputCacheParameters Page PageAsyncTask PageHandlerFactory PageParser PageParserFilter PageStatePersister PageTheme Pair ParseChildrenAttribute "+
    "PartialCachingAttribute PartialCachingControl PersistChildrenAttribute PersistenceModeAttribute PostBackOptions PostBackTrigger ProfileServiceManager PropertyConverter PropertyEntry RegisteredArrayDeclaration RegisteredDisposeScript RegisteredExpandoAttribute RegisteredHiddenField RegisteredScript RoleServiceManager RootBuilder ScriptBehaviorDescriptor ScriptComponentDescriptor ScriptControl ScriptControlDescriptor ScriptDescriptor ScriptManager ScriptManagerProxy ScriptReference ScriptReferenceBase ScriptReferenceCollection ScriptReferenceEventArgs ScriptResourceAttribute ScriptResourceDefinition ScriptResourceMapping ServiceReference ServiceReferenceCollection SessionPageStatePersister SimplePropertyEntry SimpleWebHandlerParser SkinBuilder StateBag StateItem StateManagedCollection StaticPartialCachingControl SupportsEventValidationAttribute TagPrefixAttribute TargetControlTypeAttribute TemplateBuilder TemplateContainerAttribute TemplateControl TemplateControlParser TemplateInstanceAttribute TemplateParser TemplatePropertyEntry ThemeableAttribute ThemeProvider Timer ToolboxDataAttribute Triplet UpdatePanel UpdatePanelControlTrigger UpdatePanelTrigger UpdatePanelTriggerCollection UpdateProgress UrlPropertyAttribute UserControl UserControlControlBuilder ValidationPropertyAttribute ValidatorCollection VerificationAttribute ViewStateException ViewStateModeByIdAttribute WebResourceAttribute WebServiceParser XhtmlTextWriter XPathBinder IAttributeAccessor IAutoFieldGenerator IBindableControl IBindableTemplate ICallbackEventHandler ICheckBoxControl IControlBuilderAccessor IControlDesignerAccessor IDataBindingsAccessor IDataItemContainer IDataKeysControl IDataSource IDataSourceViewSchemaAccessor IEditableTextControl IExpressionsAccessor IExtenderControl IFilterResolutionService IHierarchicalDataSource IHierarchicalEnumerable IHierarchyData INamingContainer INavigateUIData IParserAccessor IPostBackDataHandler IPostBackEventHandler IResourceUrlGenerator IScriptControl IStateFormatter IStateManager IStyleSheet ITemplate ITextControl IThemeResolutionService IUrlResolutionService IUserControlDesignerAccessor IUserControlTypeResolutionService IValidator BuildMethod BuildTemplateMethod ControlSkinDelegate DataSourceViewOperationCallback DataSourceViewSelectCallback ExtractTemplateValuesMethod ImageClickEventHandler RenderMethod AjaxFrameworkMode ClientIDMode CodeConstructType CompilationMode ConflictOptions DataSourceCacheExpiry DataSourceCapabilities DataSourceOperation HtmlTextWriterAttribute HtmlTextWriterStyle HtmlTextWriterTag OutputCacheLocation PersistenceMode RegisteredScriptType ScriptMode TemplateInstance UpdatePanelRenderMode UpdatePanelUpdateMode VerificationConditionalOperator VerificationReportLevel VerificationRule ViewStateEncryptionMode ViewStateMode VirtualReferenceType XhtmlMobileDocType AccessDataSource AccessDataSourceView AdCreatedEventArgs AdRotator AssociatedControlConverter AuthenticateEventArgs AutoGeneratedField AutoGeneratedFieldProperties BaseCompareValidator BaseDataBoundControl BaseDataList BaseValidator BoundColumn BoundField BulletedList BulletedListEventArgs Button ButtonColumn ButtonField ButtonFieldBase Calendar CalendarDay ChangePassword CheckBox CheckBoxField CheckBoxList CircleHotSpot CommandEventArgs CommandField CompareValidator CompleteWizardStep CompositeControl CompositeDataBoundControl Content ContentPlaceHolder ContextDataSource ContextDataSourceContextData ContextDataSourceView ControlIDConverter ControlParameter ControlPropertyNameConverter CookieParameter CreateUserErrorEventArgs CreateUserWizard CreateUserWizardStep CustomValidator DataBoundControl DataControlCommands DataControlField DataControlFieldCell DataControlFieldCollection DataControlFieldHeaderCell DataGrid DataGridColumn DataGridColumnCollection DataGridCommandEventArgs DataGridItem DataGridItemCollection DataGridItemEventArgs DataGridPageChangedEventArgs DataGridPagerStyle DataGridSortCommandEventArgs DataKey DataKeyArray DataKeyCollection DataList DataListCommandEventArgs DataListItem DataListItemCollection "+
	"DataListItemEventArgs DataPager DataPagerCommandEventArgs DataPagerField DataPagerFieldCollection DataPagerFieldCommandEventArgs DataPagerFieldItem DayRenderEventArgs DetailsView DetailsViewCommandEventArgs DetailsViewDeletedEventArgs DetailsViewDeleteEventArgs DetailsViewInsertedEventArgs DetailsViewInsertEventArgs DetailsViewModeEventArgs DetailsViewPageEventArgs DetailsViewPagerRow DetailsViewRow DetailsViewRowCollection DetailsViewUpdatedEventArgs DetailsViewUpdateEventArgs DropDownList EditCommandColumn EmbeddedMailObject EmbeddedMailObjectsCollection EntityDataSource EntityDataSourceChangedEventArgs EntityDataSourceChangingEventArgs EntityDataSourceContextCreatedEventArgs EntityDataSourceContextCreatingEventArgs EntityDataSourceContextDisposingEventArgs EntityDataSourceSelectedEventArgs EntityDataSourceSelectingEventArgs EntityDataSourceValidationException EntityDataSourceView FileUpload FontInfo FontNamesConverter FontUnitConverter FormParameter FormView FormViewCommandEventArgs FormViewDeletedEventArgs FormViewDeleteEventArgs FormViewInsertedEventArgs FormViewInsertEventArgs FormViewModeEventArgs FormViewPageEventArgs FormViewPagerRow FormViewRow FormViewUpdatedEventArgs FormViewUpdateEventArgs GridView GridViewCancelEditEventArgs GridViewCommandEventArgs GridViewDeletedEventArgs GridViewDeleteEventArgs GridViewEditEventArgs GridViewPageEventArgs GridViewRow GridViewRowCollection GridViewRowEventArgs GridViewSelectEventArgs GridViewSortEventArgs GridViewUpdatedEventArgs GridViewUpdateEventArgs HiddenField HierarchicalDataBoundControl HotSpot HotSpotCollection HyperLink HyperLinkColumn HyperLinkControlBuilder HyperLinkField Image ImageButton ImageField ImageMap ImageMapEventArgs Label LabelControlBuilder LinkButton LinkButtonControlBuilder LinqDataSource LinqDataSourceContextEventArgs LinqDataSourceDeleteEventArgs LinqDataSourceDisposeEventArgs LinqDataSourceInsertEventArgs LinqDataSourceSelectEventArgs LinqDataSourceStatusEventArgs LinqDataSourceUpdateEventArgs LinqDataSourceValidationException LinqDataSourceView ListBox ListControl ListItem ListItemCollection ListItemControlBuilder ListView ListViewCancelEventArgs ListViewCommandEventArgs ListViewDataItem ListViewDeletedEventArgs ListViewDeleteEventArgs ListViewEditEventArgs ListViewInsertedEventArgs ListViewInsertEventArgs ListViewItem ListViewItemEventArgs ListViewPagedDataSource ListViewSelectEventArgs ListViewSortEventArgs ListViewUpdatedEventArgs ListViewUpdateEventArgs Literal LiteralControlBuilder Localize Login LoginCancelEventArgs LoginName LoginStatus LoginView MailDefinition MailMessageEventArgs Menu MenuEventArgs MenuItem MenuItemBinding MenuItemBindingCollection MenuItemCollection MenuItemStyle MenuItemStyleCollection MenuItemTemplateContainer MonthChangedEventArgs MultiView MultiViewControlBuilder NextPreviousPagerField NumericPagerField ObjectDataSource ObjectDataSourceDisposingEventArgs ObjectDataSourceEventArgs ObjectDataSourceFilteringEventArgs ObjectDataSourceMethodEventArgs ObjectDataSourceSelectingEventArgs ObjectDataSourceStatusEventArgs ObjectDataSourceView PagedDataSource PageEventArgs PagePropertiesChangingEventArgs PagerSettings Panel PanelStyle Parameter ParameterCollection PasswordRecovery PlaceHolder PlaceHolderControlBuilder PolygonHotSpot ProfileParameter QueryableDataSource QueryableDataSourceEditData QueryableDataSourceView QueryContext QueryCreatedEventArgs QueryExtender QueryStringParameter RadioButton RadioButtonList RangeValidator RectangleHotSpot RegularExpressionValidator Repeater RepeaterCommandEventArgs RepeaterItem RepeaterItemCollection RepeaterItemEventArgs RepeatInfo RequiredFieldValidator RoleGroup RoleGroupCollection RouteParameter SelectedDatesCollection SendMailErrorEventArgs ServerValidateEventArgs SessionParameter SiteMapDataSource SiteMapDataSourceView SiteMapHierarchicalDataSourceView SiteMapNodeItem SiteMapNodeItemEventArgs SiteMapPath SqlDataSource SqlDataSourceCommandEventArgs SqlDataSourceFilteringEventArgs SqlDataSourceSelectingEventArgs SqlDataSourceStatusEventArgs SqlDataSourceView StringArrayConverter Style "+
    "StyleCollection SubMenuStyle SubMenuStyleCollection Substitution Table Table.RowControlCollection TableCell TableCellCollection TableCellControlBuilder TableFooterRow TableHeaderCell TableHeaderRow TableItemStyle TableRow TableRow.CellControlCollection TableRowCollection TableSectionStyle TableStyle TargetConverter TemplateColumn TemplatedWizardStep TemplateField TemplatePagerField TextBox TextBoxControlBuilder TreeNode TreeNodeBinding TreeNodeBindingCollection TreeNodeCollection TreeNodeEventArgs TreeNodeStyle TreeNodeStyleCollection TreeView UnitConverter ValidatedControlConverter ValidationSummary View ViewCollection WebColorConverter WebControl Wizard WizardNavigationEventArgs WizardStep WizardStepBase WizardStepCollection WizardStepControlBuilder Xml XmlBuilder XmlDataSource XmlDataSourceView XmlHierarchicalDataSourceView FontUnit Unit IButtonControl ICallbackContainer ICompositeControlDesignerAccessor IDataBoundControl IDataBoundItemControl IDataBoundListControl IFieldControl IPageableItemContainer IPersistedSelector IPostBackContainer IQueryableDataSource IRepeatInfoUser AdCreatedEventHandler AuthenticateEventHandler BulletedListEventHandler CommandEventHandler CreateUserErrorEventHandler DataGridCommandEventHandler DataGridItemEventHandler DataGridPageChangedEventHandler DataGridSortCommandEventHandler DataListCommandEventHandler DataListItemEventHandler DayRenderEventHandler DetailsViewCommandEventHandler DetailsViewDeletedEventHandler DetailsViewDeleteEventHandler DetailsViewInsertedEventHandler DetailsViewInsertEventHandler DetailsViewModeEventHandler DetailsViewPageEventHandler DetailsViewUpdatedEventHandler DetailsViewUpdateEventHandler FormViewCommandEventHandler FormViewDeletedEventHandler FormViewDeleteEventHandler FormViewInsertedEventHandler FormViewInsertEventHandler FormViewModeEventHandler FormViewPageEventHandler FormViewUpdatedEventHandler FormViewUpdateEventHandler GridViewCancelEditEventHandler GridViewCommandEventHandler GridViewDeletedEventHandler GridViewDeleteEventHandler GridViewEditEventHandler GridViewPageEventHandler GridViewRowEventHandler GridViewSelectEventHandler GridViewSortEventHandler GridViewUpdatedEventHandler GridViewUpdateEventHandler ImageMapEventHandler LoginCancelEventHandler MailMessageEventHandler MenuEventHandler MonthChangedEventHandler ObjectDataSourceDisposingEventHandler ObjectDataSourceFilteringEventHandler ObjectDataSourceMethodEventHandler ObjectDataSourceObjectEventHandler ObjectDataSourceSelectingEventHandler ObjectDataSourceStatusEventHandler RepeaterCommandEventHandler RepeaterItemEventHandler SendMailErrorEventHandler ServerValidateEventHandler SiteMapNodeItemEventHandler SqlDataSourceCommandEventHandler SqlDataSourceFilteringEventHandler SqlDataSourceSelectingEventHandler SqlDataSourceStatusEventHandler TreeNodeEventHandler WizardNavigationEventHandler AutoCompleteType BorderStyle BulletedListDisplayMode BulletStyle ButtonColumnType ButtonType CalendarSelectionMode ContentDirection DataBoundControlMode DataControlCellType DataControlRowState DataControlRowType DayNameFormat DetailsViewMode FirstDayOfWeek FontSize FormViewMode GridLines HorizontalAlign HotSpotMode ImageAlign InsertItemPosition ListItemType ListSelectionMode ListViewCancelMode ListViewItemType LiteralMode LoginFailureAction LoginTextLayout LogoutAction MenuRenderingMode NextPrevFormat Orientation PagerButtons PagerMode PagerPosition PathDirection RepeatDirection RepeatLayout ScrollBars SiteMapNodeItemType SortDirection SqlDataSourceCommandType SqlDataSourceMode TableCaptionAlign TableHeaderScope TableRowSection TextAlign TextBoxMode TitleFormat TreeNodeSelectAction TreeNodeTypes TreeViewImageSet UnitType ValidationCompareOperator ValidationDataType ValidationSummaryDisplayMode ValidatorDisplay VerticalAlign WizardStepType Constraint ConstraintCollection ConstraintException DataColumn DataColumnChangeEventArgs DataColumnCollection DataException DataRelation DataRelationCollection DataRow DataRowBuilder DataRowChangeEventArgs DataRowCollection DataRowComparer DataRowComparer DataRowExtensions DataRowView "+	
	"DataSet DataSetSchemaImporterExtension DataSysDescriptionAttribute DataTable DataTableClearEventArgs DataTableCollection DataTableExtensions DataTableNewRowEventArgs DataTableReader DataView DataViewManager DataViewSetting DataViewSettingCollection DBConcurrencyException DeletedRowInaccessibleException DuplicateNameException EntityCommandCompilationException EntityCommandExecutionException EntityException EntityKey EntityKeyMember EntitySqlException EnumerableRowCollection EnumerableRowCollection EnumerableRowCollectionExtensions EvaluateException FillErrorEventArgs ForeignKeyConstraint InRowChangingEventException InternalDataCollectionBase InvalidCommandTreeException InvalidConstraintException InvalidExpressionException MappingException MergeFailedEventArgs MetadataException MissingPrimaryKeyException NoNullAllowedException ObjectNotFoundException OperationAbortedException OptimisticConcurrencyException OrderedEnumerableRowCollection PropertyCollection ProviderIncompatibleException ReadOnlyException RowNotInTableException StateChangeEventArgs StatementCompletedEventArgs StrongTypingException SyntaxErrorException TypedDataSetGenerator TypedDataSetGeneratorException TypedTableBase TypedTableBaseExtensions UniqueConstraint UpdateException VersionNotFoundException IColumnMapping IColumnMappingCollection IDataAdapter IDataParameter IDataParameterCollection IDataReader IDataRecord IDbCommand IDbConnection IDbDataAdapter IDbDataParameter IDbTransaction IExtendedDataRecord ITableMapping ITableMappingCollection DataColumnChangeEventHandler DataRowChangeEventHandler DataTableClearEventHandler DataTableNewRowEventHandler FillErrorEventHandler MergeFailedEventHandler StateChangeEventHandler StatementCompletedEventHandler AcceptRejectRule CommandBehavior CommandType ConflictOption ConnectionState DataRowAction DataRowState DataRowVersion DataSetDateTime DataViewRowState DbType EntityState IsolationLevel KeyRestrictionBehavior LoadOption MappingType MissingMappingAction MissingSchemaAction ParameterDirection PropertyAttributes Rule SchemaSerializationMode SchemaType SerializationFormat SqlDbType StatementType UpdateRowSource UpdateStatus XmlReadMode XmlWriteMode SqlBulkCopy SqlBulkCopyColumnMapping SqlBulkCopyColumnMappingCollection SqlClientFactory SqlClientMetaDataCollectionNames SqlClientPermission SqlClientPermissionAttribute SqlCommand SqlCommandBuilder SqlConnection SqlConnectionStringBuilder SqlDataAdapter SqlDataReader SQLDebugging SqlDependency SqlError SqlErrorCollection SqlException SqlInfoMessageEventArgs SqlNotificationEventArgs SqlParameter SqlParameterCollection SqlRowsCopiedEventArgs SqlRowUpdatedEventArgs SqlRowUpdatingEventArgs SqlTransaction OnChangeEventHandler SqlInfoMessageEventHandler SqlRowsCopiedEventHandler SqlRowUpdatedEventHandler SqlRowUpdatingEventHandler ApplicationIntent SortOrder SqlBulkCopyOptions SqlNotificationInfo SqlNotificationSource SqlNotificationType AmbiguousMatchException Assembly AssemblyAlgorithmIdAttribute AssemblyCompanyAttribute AssemblyConfigurationAttribute AssemblyCopyrightAttribute AssemblyCultureAttribute AssemblyDefaultAliasAttribute AssemblyDelaySignAttribute AssemblyDescriptionAttribute AssemblyFileVersionAttribute AssemblyFlagsAttribute AssemblyInformationalVersionAttribute AssemblyKeyFileAttribute AssemblyKeyNameAttribute AssemblyName AssemblyNameProxy AssemblyProductAttribute AssemblyTitleAttribute AssemblyTrademarkAttribute AssemblyVersionAttribute Binder ConstructorInfo CustomAttributeData CustomAttributeFormatException DefaultMemberAttribute EventInfo ExceptionHandlingClause FieldInfo InvalidFilterCriteriaException LocalVariableInfo ManifestResourceInfo MemberInfo MethodBase MethodBody MethodInfo Missing Module ObfuscateAssemblyAttribute ObfuscationAttribute ParameterInfo Pointer PropertyInfo ReflectionTypeLoadException StrongNameKeyPair TargetException TargetInvocationException TargetParameterCountException TypeDelegator CustomAttributeNamedArgument CustomAttributeTypedArgument InterfaceMapping ParameterModifier ICustomAttributeProvider IReflect MemberFilter "+
	"ModuleResolveEventHandler TypeFilter AssemblyNameFlags BindingFlags CallingConventions EventAttributes ExceptionHandlingClauseOptions FieldAttributes GenericParameterAttributes ImageFileMachine MemberTypes MethodAttributes MethodImplAttributes ParameterAttributes PortableExecutableKinds ProcessorArchitecture PropertyAttributes ResourceAttributes ResourceLocation TypeAttributes NameTable UniqueId XmlAttribute XmlAttributeCollection XmlBinaryReaderSession XmlBinaryWriterSession XmlCDataSection XmlCharacterData XmlComment XmlConvert XmlDataDocument XmlDeclaration XmlDictionary XmlDictionaryReader XmlDictionaryReaderQuotas XmlDictionaryString XmlDictionaryWriter XmlDocument XmlDocumentFragment XmlDocumentType XmlElement XmlEntity XmlEntityReference XmlException XmlImplementation XmlLinkedNode XmlNamedNodeMap XmlNamespaceManager XmlNameTable XmlNode XmlNodeChangedEventArgs XmlNodeList XmlNodeReader XmlNotation XmlParserContext XmlProcessingInstruction XmlQualifiedName XmlReader XmlReaderSettings XmlResolver XmlSecureResolver XmlSignificantWhitespace XmlText XmlTextReader XmlTextWriter XmlUrlResolver XmlValidatingReader XmlWhitespace XmlWriter XmlWriterSettings IFragmentCapableXmlDictionaryWriter IHasXmlNode IStreamProvider IXmlBinaryReaderInitializer IXmlBinaryWriterInitializer IXmlDictionary IXmlLineInfo IXmlMtomReaderInitializer IXmlMtomWriterInitializer IXmlNamespaceResolver IXmlTextReaderInitializer IXmlTextWriterInitializer OnXmlDictionaryReaderClose XmlNodeChangedEventHandler ConformanceLevel DtdProcessing EntityHandling Formatting NamespaceHandling NewLineHandling ReadState ValidationType WhitespaceHandling WriteState XmlDateTimeSerializationMode XmlNamespaceScope XmlNodeChangedAction XmlNodeOrder XmlNodeType XmlOutputMethod XmlSpace XmlTokenizedType"
	),
    hooks: {
      "@": function(stream, state) {
        if (stream.eat('"')) {
          state.tokenize = tokenAtString;
          return tokenAtString(stream, state);
        }
        stream.eatWhile(/[\w\$_]/);
        return "meta";
      }, "#" :cppHook
    }
  });
  CodeMirror.defineMIME("text/x-groovy", {
    name: "clike",
    keywords: words("abstract as assert boolean break byte case catch char class const continue def default " +
                    "do double else enum extends final finally float for goto if implements import " +
                    "in instanceof int interface long native new package property private protected public " +
                    "return short static strictfp super switch synchronized this throw throws transient " +
                    "try void volatile while"),
    atoms: words("true false null"),
    hooks: {
      "@": function(stream, state) {
        stream.eatWhile(/[\w\$_]/);
        return "meta";
      }
    }
  });
}());
